@@ -804,7 +804,7 @@ func doApi10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 
 	for key := range clusterChanged {
 		switch key {
-		case "acme.agree_tos", "acme.ca_url", "acme.challenge", "acme.domain", "acme.email", "acme.provider", "acme.provider.environment", "acme.provider.resolvers":
+		case "acme.agree_tos", "acme.ca_url", "acme.challenge", "acme.domain", "acme.email", "acme.provider", "acme.provider.environment", "acme.provider.resolvers", "acme.http.port":
 			acmeChanged = true
 
 		case "cluster.images_minimal_replica":
@@ -878,6 +878,10 @@ func doApi10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 	// cluster.https_address).
 	value, ok := nodeChanged["core.https_address"]
 	if ok {
+		if value == "" && s.OS.IncusOS != nil {
+			return errors.New("Cannot unset HTTPS address on an IncusOS system")
+		}
+
 		err := s.Endpoints.NetworkUpdateAddress(value)
 		if err != nil {
 			return err
