@@ -1338,6 +1338,12 @@ func clusterNodesPost(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(err)
 	}
 
+	// Quick checks.
+	err = validate.IsAPIName(req.ServerName, false)
+	if err != nil {
+		return response.BadRequest(fmt.Errorf("Invalid cluster member name: %w", err))
+	}
+
 	if !s.ServerClustered {
 		return response.BadRequest(errors.New("This server is not clustered"))
 	}
@@ -1719,6 +1725,41 @@ func updateClusterNode(s *state.State, gateway *cluster.Gateway, r *http.Request
 		return response.BadRequest(errors.New("Cluster members need to belong to at least one group"))
 	}
 
+	// Prevent assigning all nodes the 'database-client' role.
+	clientNodes := 0
+	nodesCount := 0
+	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+		nodesCount, err = tx.GetNodesCount(ctx)
+		if err != nil {
+			return fmt.Errorf("Failed loading nodes count: %w", err)
+		}
+
+		nodes, err := tx.GetNodes(ctx)
+		if err != nil {
+			return fmt.Errorf("Failed loading nodes: %w", err)
+		}
+
+		for _, n := range nodes {
+			// Ignore the node currently being updated.
+			if n.Name == member.Name {
+				continue
+			}
+
+			if slices.Contains(n.Roles, db.ClusterRoleDatabaseClient) {
+				clientNodes += 1
+			}
+		}
+
+		return nil
+	})
+	if err != nil {
+		return response.SmartError(err)
+	}
+
+	if clientNodes+1 >= nodesCount {
+		return response.BadRequest(errors.New("Assigning the 'database-client' role to all nodes is not allowed"))
+	}
+
 	// Convert the roles.
 	newRoles := make([]db.ClusterRole, 0, len(req.Roles))
 	for _, role := range req.Roles {
@@ -1916,6 +1957,13 @@ func clusterNodePost(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(err)
 	}
 
+	// Quick checks.
+	err = validate.IsAPIName(req.ServerName, false)
+	if err != nil {
+		return response.BadRequest(fmt.Errorf("Invalid cluster member name: %w", err))
+	}
+
+	// Perform the rename.
 	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
 		return tx.RenameNode(ctx, memberName, req.ServerName)
 	})
