@@ -17,6 +17,7 @@ import (
 	"github.com/lxc/incus/v6/internal/server/instance/instancetype"
 	"github.com/lxc/incus/v6/shared/api"
 	"github.com/lxc/incus/v6/shared/osarch"
+	"github.com/lxc/incus/v6/shared/util"
 )
 
 // ImageSourceProtocol maps image source protocol codes to human-readable names.
@@ -767,9 +768,18 @@ func (c *ClusterTx) UpdateImageLastUseDate(ctx context.Context, projectName stri
 
 // SetImageCachedAndLastUseDate sets the cached and last_use_date field of the image with the given fingerprint.
 func (c *ClusterTx) SetImageCachedAndLastUseDate(ctx context.Context, projectName string, fingerprint string, lastUsed time.Time) error {
+	enabled, err := cluster.ProjectHasImages(ctx, c.tx, projectName)
+	if err != nil {
+		return fmt.Errorf("Check if project has images: %w", err)
+	}
+
+	if !enabled {
+		projectName = api.ProjectDefaultName
+	}
+
 	stmt := `UPDATE images SET cached=1, last_use_date=? WHERE fingerprint=? AND project_id = (SELECT id FROM projects WHERE name = ? LIMIT 1)`
 
-	_, err := c.tx.ExecContext(ctx, stmt, lastUsed, fingerprint, projectName)
+	_, err = c.tx.ExecContext(ctx, stmt, lastUsed, fingerprint, projectName)
 
 	return err
 }
@@ -943,19 +953,45 @@ func (c *ClusterTx) CreateImage(ctx context.Context, project string, fp string,
 	// If these projects also have features.profiles=true, their default profiles should be associated
 	// with all created images.
 	if imageProject == "default" {
-		_, err = c.tx.ExecContext(ctx,
-			`INSERT OR IGNORE INTO images_profiles(image_id, profile_id)
-					SELECT ?, profiles.id FROM profiles
-						JOIN projects_config AS t1 ON t1.project_id = profiles.project_id
-							AND t1.key = "features.images"
-							AND t1.value = "false"
-						JOIN projects_config AS t2 ON t2.project_id = profiles.project_id
-							AND t2.key = "features.profiles"
-							AND t2.value = "true"
-						WHERE profiles.name = "default"`, id)
+		allProjects, err := cluster.GetProjects(ctx, c.tx)
 		if err != nil {
 			return err
 		}
+
+		projects := []*api.Project{}
+		for _, p := range allProjects {
+			project, err := p.ToAPI(ctx, c.tx)
+			if err != nil {
+				return err
+			}
+
+			// Select the default project and projects with 'features.images' disabled and 'features.profiles' enabled.
+			if (util.IsFalse(project.Config["features.images"]) && util.IsTrue(project.Config["features.profiles"])) || project.Name == api.ProjectDefaultName {
+				projects = append(projects, project)
+			}
+		}
+
+		pIDs := []int{}
+		for _, p := range projects {
+			dbProfiles, err := cluster.GetProfilesIfEnabled(ctx, c.tx, p.Name, []string{"default"})
+			if err != nil {
+				return err
+			}
+
+			if len(dbProfiles) != 1 {
+				return fmt.Errorf("Failed to find default profile in project %q", project)
+			}
+
+			pIDs = append(pIDs, dbProfiles[0].ID)
+		}
+
+		sql = `INSERT OR IGNORE INTO images_profiles (image_id, profile_id) VALUES (?, ?)`
+		for _, profileID := range pIDs {
+			_, err = c.tx.ExecContext(ctx, sql, id, profileID)
+			if err != nil {
+				return err
+			}
+		}
 	}
 
 	_, err = c.tx.ExecContext(ctx, "INSERT INTO images_nodes(image_id, node_id) VALUES(?, ?)", id, c.nodeID)
