@@ -62,6 +62,18 @@ func (d *lvm) load() error {
 		return nil
 	}
 
+	// Handle IncusOS services.
+	if d.state.OS.IncusOS != nil {
+		ok, err := d.state.OS.IncusOS.IsServiceEnabled("lvm")
+		if err != nil {
+			return err
+		}
+
+		if !ok {
+			return errors.New("IncusOS service \"lvm\" isn't currently enabled")
+		}
+	}
+
 	// Validate the required binaries.
 	tools := []string{"lvm"}
 	if d.clustered {
@@ -144,6 +156,7 @@ func (d *lvm) Info() Info {
 		VolumeMultiNode:              d.isRemote(),
 		BlockBacking:                 true,
 		RunningCopyFreeze:            true,
+		SameSource:                   d.isRemote(),
 		DirectIO:                     true,
 		IOUring:                      true,
 		MountedRoot:                  false,
@@ -373,8 +386,15 @@ func (d *lvm) Create() error {
 			}
 		}
 	} else {
+		// Calculate the metadata size (if provided).
+		metadataSizeBytes, err := d.roundedSizeBytesString(d.config["lvm.metadata_size"])
+		if err != nil {
+			return fmt.Errorf("Invalid lvm.metadata_size: %w", err)
+		}
+
 		// Create physical volume if doesn't exist.
-		if !pvExists {
+		// This is done to provide better errors and revert when possible.
+		if !pvExists && !d.clustered {
 			// This is an internal error condition which should never be hit.
 			if pvName == "" {
 				return errors.New("No name for physical volume detected")
@@ -382,11 +402,6 @@ func (d *lvm) Create() error {
 
 			args := []string{}
 
-			metadataSizeBytes, err := d.roundedSizeBytesString(d.config["lvm.metadata_size"])
-			if err != nil {
-				return fmt.Errorf("Invalid lvm.metadata_size: %w", err)
-			}
-
 			if metadataSizeBytes > 0 {
 				args = append(args, "--metadatasize", fmt.Sprintf("%db", metadataSizeBytes))
 			}
@@ -406,9 +421,13 @@ func (d *lvm) Create() error {
 
 		if d.clustered {
 			args = append(args, "--shared")
+
+			if metadataSizeBytes > 0 {
+				args = append(args, "--metadatasize", fmt.Sprintf("%db", metadataSizeBytes))
+			}
 		}
 
-		_, err := subprocess.TryRunCommand("vgcreate", args...)
+		_, err = subprocess.TryRunCommand("vgcreate", args...)
 		if err != nil {
 			return err
 		}
@@ -438,7 +457,7 @@ func (d *lvm) Create() error {
 			d.logger.Debug("Thin pool created", logger.Ctx{"vg_name": d.config["lvm.vg_name"], "thinpool_name": d.thinpoolName()})
 
 			reverter.Add(func() {
-				_ = d.removeLogicalVolume(d.lvmDevPath(d.config["lvm.vg_name"], "", "", d.thinpoolName()))
+				_ = d.removeLogicalVolume(d.lvmPath(d.config["lvm.vg_name"], "", "", d.thinpoolName()))
 			})
 		} else if d.config["size"] != "" {
 			return errors.New("Cannot specify size when using an existing thin pool")
@@ -512,7 +531,7 @@ func (d *lvm) Delete(op *operations.Operation) error {
 					} else if thinVolCount == 0 && lvCount > 1 {
 						// Otherwise, if the thin pool is empty but the volume group has
 						// other volumes, then just remove the thin pool volume.
-						err = d.removeLogicalVolume(d.lvmDevPath(d.config["lvm.vg_name"], "", "", d.thinpoolName()))
+						err = d.removeLogicalVolume(d.lvmPath(d.config["lvm.vg_name"], "", "", d.thinpoolName()))
 						if err != nil {
 							return fmt.Errorf("Failed to delete thin pool %q from volume group %q: %w", d.thinpoolName(), d.config["lvm.vg_name"], err)
 						}
@@ -695,7 +714,7 @@ func (d *lvm) Update(changedConfig map[string]string) error {
 		}
 
 		if d.usesThinpool() {
-			lvPath := d.lvmDevPath(d.config["lvm.vg_name"], "", "", d.thinpoolName())
+			lvPath := d.lvmPath(d.config["lvm.vg_name"], "", "", d.thinpoolName())
 
 			// Use the remaining space in the volume group.
 			_, err = subprocess.RunCommand("lvresize", "-f", "-l", "+100%FREE", lvPath)
@@ -803,7 +822,7 @@ func (d *lvm) GetResources() (*api.ResourcesStoragePool, error) {
 	// Thinpools will always report zero free space on the volume group, so calculate approx
 	// used space using the thinpool logical volume allocated (data and meta) percentages.
 	if d.usesThinpool() {
-		volDevPath := d.lvmDevPath(d.config["lvm.vg_name"], "", "", d.thinpoolName())
+		volDevPath := d.lvmPath(d.config["lvm.vg_name"], "", "", d.thinpoolName())
 		totalSize, usedSize, err := d.thinPoolVolumeUsage(volDevPath)
 		if err != nil {
 			return nil, err
