@@ -34,6 +34,12 @@ var DefaultRemotes = map[string]Remote{
 type DefaultSettings struct {
 	// Default flag format for list commands.
 	ListFormat string `yaml:"list_format"`
+
+	// Preferred console type.
+	ConsoleType string `yaml:"console_type"`
+
+	// Alternative SPICE command (SOCKET will be replaced by the socket path).
+	ConsoleSpiceCommand string `yaml:"console_spice_command"`
 }
 
 // DefaultConfig returns the default configuration.
