@@ -175,6 +175,13 @@ EOF
     volume.size: 25MiB
     size: 1GiB
     lvm.vg_name: incustest-$(basename "${TEST_DIR}")-${ns}
+EOF
+        fi
+        if [ "${driver}" = "truenas" ]; then
+            cat >> "${INCUS_DIR}/preseed.yaml" << EOF
+  config:
+    source: $(truenas_host_dataset)/incustest-$(basename "${TEST_DIR}")
+    truenas.api_key: \"${INCUS_TRUENAS_API_KEY}\"
 EOF
         fi
         if [ "${driver}" = "ceph" ]; then
